/**********************************************************/
/*        Definition of the prototype of functions        */
/*        to make the alea statistic tests                */
/**********************************************************/
#ifndef _B_TEST_H__
#define _B_TEST_H__

#include "statistic_types.h"
#include "bitutils.h"

#define MAURER_MAX 16

/* Datas for the Maurer (taken from MOV pp. 184) */
static double Maurer_Data[16][2] ={
  {0.7326495,0.690},
  {1.5374383,1.338},
  {2.4016068,1.901},
  {3.3112247,2.358},
  {4.2534266,2.705},
  {5.2177052,2.954},
  {6.1962507,3.125},
  {7.1836656,3.238},
  {8.1764249,3.311},
  {9.1723243,3.356},
  {10.170032,3.384},
  {11.168765,3.401},
  {12.168765,3.410},
  {13.167693,3.416},
  {14.167488,3.419},
  {15.167379,3.421}
};

/* Frequency test */
/* this function computes the Frequency test for words of length guiven by the parameter length */
extern struct statistic Frequency(struct BitField *Field, int length);

/* this function computes the KolmogorovSmirvov test for the Frequency test */
extern struct test Statistic_Frequency(struct test_parameters parameters);

/* Correlation test */
/* this function computes the Correlation test for words of length guiven by the parameter length */
extern struct statistic Correlation(struct BitField *Field, int length, int space);

/* this function computes the KolmogorovSmirvov test for the Correlation test */
extern struct test Statistic_Correlation(struct test_parameters parameters);

/* Spectral test */
/* this function computes the Spectral test for words of length given by the parameter length */
extern bsize_t *Spectral_Sample(struct BitField *Field, int order);

/* this function compute the KolmogorovSmirvov test for the Spectral test */
extern struct test Statistic_Spectral(struct test_parameters parameters);

/* Run and Gap test */
/* this function computes the Run test for words of length given by the parameter length */
extern struct statistic Run(struct BitField *Field, int length, word_t mbit);

/* this function computes the KolmogorovSmirvov test for the Run test */
extern struct test Statistic_Run(struct test_parameters parameters);

/* Maurer test */
/* this function computes the Maurer test for words of length given by the parameter length */
extern struct statistic Maurer(struct BitField *Field, int length);

/* this function computes the KolmogorovSmirnov test for the Maurer test */
extern struct test Statistic_Maurer(struct test_parameters parameters);

/* Shanon Entropy */
/* this function computes the shanon entropy for words of length n */
extern struct test Shannon_Entropy(struct test_parameters parameters);

/* Linear Complexity */
/* this function computes a chi2 distribution related to the linear complexity of bit sequence */
struct statistic Linear_Complexity_Chi2(struct BitField *Field, bsize_t nb_chunks, bsize_t nb_parts);

/* this function computes the linear complexity test of bit sequence */
extern struct test Statistic_Linear_Complexity(struct test_parameters parameters);


/* Collision */
/* this function computes the collision test for words of length given by the parameter length */
extern struct statistic Collision(struct BitField *Field, bsize_t flength, bsize_t length);

/* Smallest Collision */
/* This function computes the smallest length for which there is no occurence of a collision
*/
extern struct test Smallest_No_Collision(struct test_parameters parameters);

#endif /*  __TEST_H__ */
