/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/statistic_tests.h"
#include "../include/bitutils.h"
#include "../include/special_functions.h"


/*
proto: struct statistic Collision(struct BitField *Field, bsize_t flength, bsize_t length)
type: low 
comment: This function computes the collision test for words of length given by the parameter length
*/
struct statistic Collision(struct BitField *Field, bsize_t flength, bsize_t length)

{
  bsize_t i,j, k, col, maxcol;
  struct BitField Field2;
  struct node *tab;
  char *ptr;
  unsigned int the, last, chunk;
  word_t bit;
  struct statistic result; 
  int memory_flag=1;

  last = 1; chunk = 1; col = 0; maxcol = 0; result.error = 0;

  BF_Create(&Field2, length, length);
  tab = (struct node *)malloc(MEMORY_CHUNK *sizeof(struct node));

  ptr = (char *) tab;
  for(k=0; k<MEMORY_CHUNK *sizeof(struct node); k++)
    {
      ptr[k]=0;
    }

  for (i=1; i<=flength-length; i=i+length)
    {
      the = 0;
      for (j=0; j<length-1; j++)
      {
	bit = BF_Take(Field, i+j);
	bit = bit & 1UL;
	if ((tab[the].next[bit] == 0) && (memory_flag == 0))
	  {
	    break;
	  }
	if ((tab[the].next[bit] == 0) && (memory_flag == 1))
	  {
	    tab[the].next[bit]=last;
	    last++;
	    if (last*sizeof(struct node) > MAX_MEMORY)
	      {
		memory_flag = 0;
		result.error = 1;
		printf("Warning: memory limit reached \n");
	      }
	    if (last*sizeof(struct node) > MEMORY_CHUNK*chunk)
	      {
		chunk++;
		tab = (struct node *) realloc(tab, MEMORY_CHUNK*sizeof(struct node)*chunk);
		ptr = (char *)tab;
		for(k=MEMORY_CHUNK*sizeof(struct node)*(chunk-1); k<MEMORY_CHUNK*sizeof(struct node)*chunk; k++)
		  {
		    ptr[k]=0;
		  }
	      }
	  }
	the = tab[the].next[bit];
      }

      if (j == length -1)
	{
	  tab[the].next[0]++; 
	  
	  
	  if (tab[the].next[0] > 1)
	    {
#ifdef STATISTICS
	      printf("************\n");
	      printf("%d \n", tab[the].next[1]);
#endif
	      col++;
	      tab[the].next[1]=i;

	      BF_Take_Field(Field,&Field2, 1, length);
#ifdef STATISTICS
	      BF_Display(&Field2, length, 16);
#endif
	    }
	}
    }

  BF_Dispose(&Field2);
  free(tab);

#ifdef STATISTICS
  printf("%d \n", col);
#endif

  result.result = col;
  return result;
}

/*
proto: struct statistic Smallest_No_Collision(struct BitField *Field)
type: low 
comment: This function computes the smallest length for which there is no occurence of a collision
*/
struct test Smallest_No_Collision(struct test_parameters parameters)

{
  long int step;
  bsize_t length;
  int flag;
  struct statistic collision;
  struct test result;

  flag = 1;

  length = (bsize_t)ceil(log((double)BITLENGTH(parameters.data)));

  step = length/2;
  while (flag)
    {

      collision = Collision(parameters.data, BITLENGTH(parameters.data), length);
      if ((collision.result == 0) && (step > 0))
	{
	  step = floor(-step/2);
	}
      if ((collision.result > 0) && (step < 0))
	{
	  step = floor(-step/2);
	}
      length = length + step;
      if (step == 0)
	{
	  flag = 0;
	}
    }

  result.z = length;
  return result;
}
