/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/statistic_tests.h"
#include "../include/special_functions.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          CORRELATION TEST
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct statistic Correlation(struct BitField *Field, int length, int space)
type: low 
comment: This function computes the Correlation test for words of length guiven by the parameter length
*/
struct statistic Correlation(struct BitField *Field, int length, int space)
{
  bsize_t i;
  bsize_t *array;
  word_t k1,k2,k;
  struct statistic result;
  
  if (length >= WORDLENGTH || length==0) 
    {
      result.error=1;
      return result;
    }
  array = (bsize_t *) calloc(1<<length,sizeof(bsize_t));

  for (i=1; i <= BITLENGTH(Field)-length-space; i = i+2*length)
  {
    k1 = BF_Take(Field,i) << (WORDLENGTH - length) >> (WORDLENGTH - length);
    k2 = BF_Take(Field,i+space) << (WORDLENGTH - length) >> (WORDLENGTH - length);
    k=k1^k2;
    array[k]=array[k]+1;
  }
  result=Equi_Chi_Carre(array,(1<<length)-1);

#if STATISTICS >= 2
  for(i=0; i< MIN((1<<length), MAXSTAT); i++)
    {
      printf("%d %d \n",i,array[i]); 
    }
#endif

  free(array);
  result.error=0;
  return result;
}

/*
proto: struct test Statistic_Correlation(struct test_parameters parameters)
type: high 
comment: This function compute the KolmogorovSmirvov test for the Correlation test
*/
struct test Statistic_Correlation(struct test_parameters parameters)
{
  struct test result;
  struct statistic chi2;
  struct BitField *array;
  double *proba;
  unsigned int nliberty,i;
  double KS, min, max;
  bsize_t offset;
  int flag;
  
  if (proba = (double *)calloc(parameters.nb_chunks, sizeof(double)))
    {
      array = BF_Split(parameters.data,parameters.nb_chunks);

      #ifdef STATISTICS
      printf("Number of chunks : %d \n", parameters.nb_chunks);
      printf("Length of a chunk in bits : %d \n", BITLENGTH((&array[0])));
      #endif

      min = HUGE_VAL; 
      max = -HUGE_VAL;
      flag=0;
      for(i=0; i<parameters.nb_chunks; i++)
	{
	  chi2= Correlation(&array[i],parameters.length, parameters.space);

#if STATISTICS >= 1
	  printf("Chunks %d, chi2 = %f\n", i, chi2.z);
#endif 

	  proba[i]=chi2.z;
	  flag= chi2.warning | flag;
	  if (min > proba[i])
	    min = proba[i];
	  if (max < proba[i])
	    max = proba[i];
	}
      nliberty=(1<<parameters.length) - 1;
      KS = Kolmogorov_Smirnov(proba, parameters.nb_chunks, (double (*)(double, void *))chi2_distribution, &nliberty);

      min = MIN(parameters.min,min);
      max = MAX(parameters.max,max);

      if (strcmp(parameters.file_name,"") != 0)
	{
	  plot_graph(proba, parameters.nb_chunks, parameters.file_name);
	}
      if (strcmp(parameters.file_name1,"") != 0)
	{
      	 plot_function((double (*)(double, void *))chi2_distribution, &nliberty, min, max, parameters.nb_points, parameters.file_name1);
	}
      
      free(proba);
      offset=BITLENGTH(parameters.data)%parameters.nb_chunks;
      for(i=0; i<parameters.nb_chunks+(offset != 0); i++)
	{
	  BF_Dispose(&array[i]);
	}
      free(array);
      result.min = min;
      result.max = max;
      result.z= KS;
      result.error=0;
      result.warning = flag;
      return result;
    }
  else
    {
      result.error=1;
      result.z=0;
      return result;
    }
}
