/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/statistic_tests.h"
#include "../include/special_functions.h"



/* compute (-1)^n */
bsize_t minuspow(bsize_t n)
{
  n = n & 1UL;

  if (n==1) 
    {
      return -1;
    }
  else
    {
      return 1;
    }
}


/* this function compute the sum of two bitfields with a right shift */
int BF_Sum(struct BitField *Field1, struct BitField *Field2, int l)

{
  bsize_t i;
  bsize_t h, last;
  word_t mword;
  
  h = BITLENGTH(Field2);
  if ((BITLENGTH(Field2) + l) > BITLENGTH(Field1))
    {
      Field1->l = ((BITLENGTH(Field2) + l) >> LOG2WORDLENGTH)+1;
      Field1->offset = (BITLENGTH(Field2)-1 + l) - ((Field1->l-1) << LOG2WORDLENGTH) -1;
    }

  last = h >> LOG2WORDLENGTH;
  for (i=0; i <= last; i++)
    {
      mword=Field2->data[i];
      BF_Put(Field1, l + i* WORDLENGTH, mword);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          LINEAR COMPLEXITY COMPUTATION
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct statistic Linear_Complexity(struct BitField *Field, bsize_t length)
type: low 
comment: This function compute the linear commplexite of a BitField of length length
*/
struct statistic Linear_Complexity(struct BitField *Field, bsize_t length)
{
  struct BitField poly_retro1, poly_retro2, poly_retro3, l1;
  bsize_t n, L, BL;
  word_t d;
  int m;
  struct statistic result;
  
  if (length > BITLENGTH(Field))
    length = BITLENGTH(Field);
  
  
  BF_Create(&poly_retro1, 0, length);
  BF_Create(&poly_retro2, 0, length);
  BF_Create(&poly_retro3, 0, length);
  BF_Create(&l1, 0, BITLENGTH(Field)+1);  
  
  BF_Append_Bit(&poly_retro1, 1);
  BF_Append_Bit(&poly_retro2, 1);
  
  n = 0; L=0;  m = -1;
  BF_Reverse(Field, &l1);
  BF_Take_Field(&l1, Field, 1, BITLENGTH((&l1)));

  BL = BITLENGTH(Field);
  
  while (n < length)
    {
      BF_Take_Field(Field, &l1, BL-n, L+1);
      
      d = BF_Scalar(&poly_retro1, &l1, L+1);

#ifdef STATISTICS
       printf("d= %d \n", BITLENGTH((&poly_retro1)));
#endif

      if (d ==1)
	{
	  BF_Take_Field(&poly_retro1, &poly_retro3, 1, BITLENGTH((&poly_retro1)));
	  BF_Sum(&poly_retro1, &poly_retro2, n-m+1);
	  if (L <= n/2)
	    {
	      L=n+1-L; m=n; 
	      BF_Take_Field(&poly_retro3, &poly_retro2, 1, BITLENGTH((&poly_retro3)));
	    }
	}
      n++;
    }
  result.result = BITLENGTH((&poly_retro1));

  BF_Dispose(&poly_retro1);
  BF_Dispose(&poly_retro2);
  BF_Dispose(&poly_retro3);
  
  result.error = 0;

  return result;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          LINEAR COMPLEXITY CHI 2 COMPUTATION
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct statistic Linear_Complexity(struct BitField *Field, bsize_t length)
type: low 
comment: This function compute the linear commplexite of a BitField of length length
*/
struct statistic Linear_Complexity_Chi2(struct BitField *Field, bsize_t nb_chunks, bsize_t nb_parts)
{
  struct BitField *array;
  double mu,ti;
  bsize_t length,n;
  long int res;
  unsigned int i;
  bsize_t *tabresult;
  double *tabprob;
  struct statistic result, linresult;

      array = BF_Split(Field,nb_chunks);
      tabresult = (unsigned long int *)calloc(nb_parts, sizeof(long int));
      tabprob = (double *)calloc(nb_parts, sizeof(double));

#ifdef STATISTICS
      printf("Number of chunks : %d \n", nb_chunks);
      printf("Length of a chunk in bits : %d \n", BITLENGTH((&array[0])));
#endif
      
      length = BITLENGTH((&array[0]));
      mu = (double)length/2.0+1/36*(9.0+(double)minuspow(length+1))-((double)length/3.0+2.0/9.0)/pow(2,(double)length);
      
      for(i=0; i<nb_chunks; i++)
	{
	  linresult = Linear_Complexity(&array[i], BITLENGTH((&array[i])));
	  ti = (double)minuspow(length)*((double)linresult.result-mu)+2/9.0;
	  res = (long int)floor(ti+((double)nb_parts-1)/2);

	  
	  if ((res >=0) && (res < nb_parts))
	    {
	      tabresult[res] = tabresult[res]+1;
	    }
	}

      for (i=0; i<nb_parts/2; i++)
	{
	  tabprob[i+nb_parts/2]= nb_chunks*1/pow(2,2*i);
	}
      
      for (i=0; i<=nb_parts/2; i++)
	{
	  tabprob[nb_parts/2-i]= nb_chunks*1/pow(2,2*i+1);
	}

      result = Chi_Carre(tabresult, tabprob, nb_parts-1);
      return result;		      
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          LINEAR COMPLEXITY TEST
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct test Statistic_Linear_Complexity(struct test_parameters parameters)
type: high
comment: this function is a high level version of the Linear complexity test
*/
struct test Statistic_Linear_Complexity(struct test_parameters parameters)
{
  struct test result;
  struct statistic chi2;
  struct BitField *array;
  double  *proba;
  unsigned int nliberty,i;
  double KS, min, max;
  bsize_t offset;
  int flag;

  if (proba = (double *)calloc(parameters.nb_chunks, sizeof(double)))
    {
      array = BF_Split(parameters.data,parameters.nb_chunks);

#ifdef STATISTICS
      printf("Number of chunks : %d \n", parameters.nb_chunks);
      printf("Length of a chunk in bits : %d \n", BITLENGTH((&array[0])));
#endif
      
      min = HUGE_VAL; 
      max = -HUGE_VAL;
      flag=0;
      for(i=0; i<parameters.nb_chunks; i++)
	{
	  chi2 = Linear_Complexity_Chi2(&array[i], NBL_CHUNKS, NB_PARTS);

#if STATISTICS >= 1
	  printf("Chunks %d, chi2 = %f\n", i, chi2.z);
#endif 

	  proba[i]=chi2.z;
	  flag= chi2.warning | flag;
	  if (min > proba[i])
	    min = proba[i];
	  if (max < proba[i])
	    max = proba[i];
	}
      nliberty=NB_PARTS-1;
      KS = Kolmogorov_Smirnov(proba, parameters.nb_chunks, (double (*)(double, void *))chi2_distribution, &nliberty);

      min = MIN(parameters.min,min);
      max = MAX(parameters.max,max);

      if (strcmp(parameters.file_name,"") != 0)
	{
	  plot_graph(proba, parameters.nb_chunks, parameters.file_name);
	}
      if (strcmp(parameters.file_name1,"") != 0)
	{
      	 plot_function((double (*)(double, void *))chi2_distribution, &nliberty, min, max, parameters.nb_points, parameters.file_name1);
	}

      free(proba);
      offset=BITLENGTH(parameters.data)%parameters.nb_chunks;

      for(i=0; i<parameters.nb_chunks+(offset != 0); i++)
	{
	  BF_Dispose(&array[i]);
	}
      free(array);
      result.min = min;
      result.max = max;
      result.z= KS;
      result.error=0;
      result.warning = flag;
      return result;
    }
  else
    {
      result.error=1;
      result.z=0;
      return result;
    }
}
