/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "../include/protos.h"
#include "../include/bitutils.h"
#include "../include/statistic_tests.h"
#include "../include/special_functions.h"
#include "../include/test.h"

/**********************************
 *    T E X                       *
 **********************************/
void tex(FILE *output, char *nom_test, char *title, char ***tab, int columns, int test_min, int test_max, int title_lignes)

{
  int i,j;

  fprintf(output, "\%\%%s\%\%\n", nom_test);
  fprintf(output, "\%\%%sg\%\%\n", nom_test);
  fprintf(output, "\%\%%d\%\%\n", test_min);
  fprintf(output, "\%\%%d\%\%\n", test_max);
  fprintf(output,"\\begin{tabular}{|");
  for(i=1; i<= columns; i++)
    fprintf(output,"r|");
  fprintf(output,"} \n\\hline");
  fprintf(output, "\\multicolumn{ %d }{|l|}{ %s }\\\\ \n", columns,title);
  fprintf(output, "\\hline \n");

  fprintf(output, "\\multicolumn{ %d }{|l|}{ Number of blocs %d }\\\\ \n", columns,NB_CHUNKS);
  fprintf(output, "\\hline \n");

  for(i=0; i<title_lignes; i++)
    {
      fprintf(output, "%s", tab[i][0]);
    }
  for(i=title_lignes; i<= test_max-test_min+1+title_lignes; i++)
    {
      for(j=0; j< columns-1; j++)
	{
	  fprintf(output, "%s &", tab[i][j]);
	}
      	  fprintf(output, "%s", tab[i][j]);
      fprintf(output,"\\\\ \\hline \n");
    }
  fprintf(output,"\\end{tabular} \n");
  fprintf(output, "\%\%%s\%\%\n", nom_test);
  fprintf(output,"\n");

}

/**********************************
 *    U S A G E                   *
 **********************************/
 
void usage(char *str)
{
  printf("Usage : %s name_of_a_file", str);
}

/**********************************
 *   UNSIGNED INT TO STRING       *
 **********************************/
void utostring(unsigned long int u, char *output)
{
  char tmp[255];
  int i, j;
  
  i=0;
  do
    {
      tmp[i]='0'+(u%10);
      u=u/10;
      i++;
    }
    while (u > 0);
  for(j=0; j<i; j++)
    {
      output[j]=tmp[i-1-j];
    }
  output[j]='\0';
}

/**********************************
 *   FLOAT INT TO STRING       *
 **********************************/
void ftostring(double x, char *output, int max)
{
  int i, point;
  unsigned long u;

  u = (unsigned long)floor(x);
  utostring(u,output);
  x = x - floor(x);
  point = 0;
  i =strlen(output);

  while ((x > 0) && (i <= max))
    {
      if (point == 0)
	{
	  output[i]='.';
	  point = 1;
	  i++;
	}
      x = x * 10;
      output[i]='0'+(int)floor(x);
      x = x - (int)floor(x);
      i++;
    }
  output[i]='\0';
}


/**********************************
 *   DO TEST                      *
 **********************************/
void do_test(char *nom_test, char* name_of_the_test, char **name_sequences, int test_min, int test_max, int number_sequences, FILE *plot, FILE *texoutput, struct BitField *Field, struct test_parameters parameters,struct test(*F)(struct test_parameters))

{
 char *numero, *name_output, *name_output1, *name_output2, ***texout, *title;
 int i,j;
 double min, max;
 struct test value;


 numero = (char *)malloc(255);
 name_output = (char *)malloc(255);
 name_output1 = (char *)malloc(255);
 name_output2 = (char *)malloc(255);
 
 texout = malloc(MAX_LIGNES * sizeof(size_t));

 for(i=0; i< MAX_LIGNES; i++)
   {
     texout[i]= malloc(MAX_COLUMNS* sizeof(size_t));
   }
 for(i=0; i< MAX_LIGNES; i++)
   {
     for(j=0; j< MAX_COLUMNS; j++)
	    {
	      texout[i][j]=(char *)malloc(255);
	    }
   }

 for (i=0; i< number_sequences; i++)
   {
     utostring(number_sequences+1, numero);
     strcpy(texout[i][0],"\\multicolumn{ ");
     strcat(texout[i][0], numero);
     strcat(texout[i][0], "}{|l|}{");
     strcat(texout[i][0], "Bit size of the sequence ");
     strcat(texout[i][0], name_sequences[i]);
     strcat(texout[i][0], " ");
     utostring(BITLENGTH((&Field[i])), numero);
     strcat(texout[i][0], " ");
     strcat(texout[i][0], numero);
     strcat(texout[i][0], "}\\\\ \n \\hline \n");
   }

 strcpy(texout[number_sequences][0],"Nbr of bits"); 
 for (i=0; i< number_sequences; i++)
   {
     strcpy(texout[number_sequences][i+1], name_sequences[i]);
   }
 
 /* Main loop to compute the test on each sequence to check */
 for(i=test_min; i<= test_max; i++)
   {
     /* output the plot commands */
     /* name of the output file */
	 /* name of the eps file */
     strcpy(name_output,EPS_DIR);
     strcat(name_output, nom_test);
     utostring(i, numero);
     strcat(name_output, numero);
     strcat(name_output,".eps");

     strcpy(name_output1,GRAPH_DIR);
     strcat(name_output1, nom_test);
     strcat(name_output1, numero);
     strcat(name_output1, "_theoritical");

     if (plot)
       {
	 fprintf(plot,"\n");
	 #ifdef WITH_TITLE
	 strcpy(name_output2, nom_test);
	 strcat(name_output2, " ");
	 strcat(name_output2, numero);
	 fprintf(plot,"set title \"%s\"\n", name_output2);
	 #endif
	 fprintf(plot,"set output \"%s\"\n", name_output);
	 fprintf(plot,"set terminal postscript eps color solid\n");
	 fprintf(plot,"plot \"%s\" title \"theoritical\"with lines",name_output1);
       }
     
     min = HUGE_VAL;
     max = -HUGE_VAL;

     for (j=0; j< number_sequences; j++)
       {
	 /* name of the output file */
	 strcpy(name_output,GRAPH_DIR);
	 strcat(name_output, name_sequences[j]);
	 utostring(i, numero);
	 strcat(name_output, nom_test);
	 strcat(name_output, numero);
	 
	 /* seting the parameters of the test */
	 parameters.data = &Field[j];
	 parameters.file_name = name_output;
	 parameters.min = min;
	 parameters.max = max;
	 parameters.length = i;
	
	 strcpy(name_output1,GRAPH_DIR);
	 strcat(name_output1, nom_test);
	 strcat(name_output1, numero);
	 strcat(name_output1, "_theoritical");

	 parameters.file_name1 = name_output1;
	 value = F(parameters);
	 
	 printf("%s ", nom_test);
	 printf("%s ", name_sequences[j]);
	 printf("%d = %f \n",i,value.z);

	 min = MIN(value.min,min);
	 max = MAX(value.max,max);

	 /* preparing the value for the latex page */
	 utostring(i,texout[i-test_min+number_sequences+1][0]);
	 ftostring(value.z,texout[i-test_min+number_sequences+1][j+1],6);

	 
	 /* ouput gnuplot commands */
	 if (plot)
	   fprintf(plot,",\"%s\" title \"%s\" with steps",name_output, name_sequences[j]);
       }
   }
 tex(texoutput, nom_test, name_of_the_test, texout, number_sequences+1, test_min, test_max,number_sequences);
 for(i=0; i< MAX_LIGNES; i++)
   {
     for(j=0; j< MAX_COLUMNS; j++)
       {
	 free(texout[i][j]);
       }
   }
 for(i=0; i< MAX_LIGNES; i++)
   {
     free(texout[i]);
   }

 free(texout);
 free(name_output);
 free(name_output1);
 free(name_output2);
 free(numero);
}


/**********************************
 *   MAIN                         *
 **********************************/
main(int argc, char *argv[])
{
  int i;
  struct BitField *bits;
  struct test value;
  struct test_parameters parameters;
  struct stat buf;
  FILE *plot, *texoutput;
    


  if ((argc > 0) && (argc < MAX_LIGNES))
    {
        bits = (struct BitField *)malloc((argc-1)*sizeof(struct BitField));

	for ( i=1; i<argc; i++)
	  {
	    if (BF_Open_File(&bits[i-1], argv[i]))
	    {
	      stat(argv[i], &buf); 
	      printf("Length of the file %s : %d \n", argv[i], buf.st_size);
	    }
	    else
	      {
		printf("Can not open file %s\n", argv[i]);
	      }
	  }
	
	plot = fopen(PLOT,"w");
	texoutput = fopen(TEX,"w");
	
	parameters.nb_points = NB_POINTS;
	parameters.nb_chunks = NB_CHUNKS;
	
	/* frequency multi-bit test */
#ifdef DO_FREQUENCY
      do_test("frequency", NAME_FREQUENCY, &argv[1], MIN_FREQUENCY, MAX_FREQUENCY, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Frequency);
#endif
      
      /* correlation multi-bit test */
#ifdef DO_CORRELATION
      parameters.space = SPACE;
      do_test("correlation", NAME_CORRELATION, &argv[1], MIN_CORRELATION, MAX_CORRELATION, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Correlation);
#endif
      
      /* spectral test */
#ifdef DO_SPECTRAL
      do_test("spectral", NAME_SPECTRAL, &argv[1], MIN_SPECTRAL, MAX_SPECTRAL, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Spectral);
#endif   
  
      /*  Maurer test */
#ifdef DO_MAURER
      do_test("maurer", NAME_MAURER, &argv[1], MIN_MAURER, MAX_MAURER, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Maurer);
#endif
  
      /* run test */
#ifdef DO_RUN
      parameters.bit = 1;
      do_test("run", NAME_RUN, &argv[1], MIN_RUN, MAX_RUN, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Run);
#endif
  
      /* run gap */
#ifdef DO_GAP
      parameters.bit = 0;
      do_test("gap", NAME_GAP, &argv[1], MIN_GAP, MAX_GAP, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Run);
#endif
  
      /* entropy computation */
#ifdef DO_ENTROPY
      do_test("entropy", NAME_ENTROPY, &argv[1], MIN_ENTROPY, MAX_ENTROPY, argc-1, 0, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Shannon_Entropy);
#endif

      /* linear complexity */
#ifdef DO_LINEAR_COMPLEXITY
      do_test("berlekamp", NAME_LINCOMP, &argv[1], 1, 1, argc-1, plot, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Statistic_Linear_Complexity);
#endif

      /* collision */
#ifdef DO_COLLISION
      do_test("collision", NAME_COLLISION, &argv[1], 1, 1, argc-1, 0, texoutput, bits, parameters, (struct test (*)(struct test_parameters))Smallest_No_Collision);
#endif


      free(bits);
  
      fclose(plot);
      fclose(texoutput);
    }
  else
    {
      printf("Usage: main filename \n");
    }
}


