/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/statistic_tests.h"
#include "../include/utilities.h"
#include "../include/special_functions.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          PERMUTATION TEST
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*given an array this fonction retuns the index of the maximum element */
int Maximum(bsize_t *array, int length)

{
  int i, result;
  
  result = 0;
  max = array[0];
  for(i=1; i<= length; i++)
    {
      if (max < array[i])
	{
	  max = array[i];
	  result = i;
	}
    }
  return result;
}

/* this function return an interger which characterise the order of a tuple */
unsigned int P(bsize_t *array, int length)

{
  unsigned int result;
  int last, max;
  bsize_t temp;

  last = length-1;
  result = 0;
  
  while (last > 0)
    {
      max = Maximum(array, last);
      temp = array[last];
      array[last]=array[max];
      array[max]=temp;
      result = (last+1)*result+max;
      last--;
    }
}


/* this function compute the Frequency test for words of length given by the parameter length */
double Permutation(struct BitField *Field, int length)

{
  bsize_t i;
  bsize_t *array;
  word_t k1,k2,k;
  double chi2;
  
  if (length >= 32 || length==0) 
    return 0;

  array = (bsize_t *) calloc(1<<length,sizeof(bsize_t));

  for (i=1; i <= BITLENGTH(Field)-length; i = i+length)
  {
    k1 = BF_Take(Field,i) << (WORDLENGTH - length) >> (WORDLENGTH - length);
    k2 = BF_Take(Field,i+length) << (WORDLENGTH - length) >> (WORDLENGTH - length);
    k=k1^k2;
    array[k]=array[k]+1;
  }
  chi2=Equi_Chi_Carre(array,(1<<length)-1);

#ifdef VERBOSE
  for(i=0; i< (1<<length); i++)
    {
      printf("%d %d \n",i,array[i]); 
    }
#endif

  free(array);
  return chi2;
}

/* this function compute the KolmogorovSmirvov test for the Correlation test */
double Statistic_Correlation(struct BitField *Field, bsize_t nb_chunks, bsize_t length, char *file_name)
{
  struct BitField *array;
  double *proba;
  unsigned int nliberty,i;
  double KS;
  bsize_t offset;

  if (proba = (double *)calloc(nb_chunks, sizeof(double)))
    {
      array = BF_Split(Field,nb_chunks);
      for(i=0; i<nb_chunks; i++)
	{
	  proba[i]= Correlation(&array[i],length);
	}
      if (strcmp(file_name,"") != 0)
	{
	  plot_graph(proba, nb_chunks, file_name);
	}
      nliberty=nb_chunks - 1;
      KS = Kolmogorov_Smirnov(proba, nb_chunks, (double (*)(double, void *))chi2_distribution, &nliberty);
      free(proba);
      offset=BITLENGTH(Field)%nb_chunks;
      for(i=0; i<nb_chunks+(offset != 0); i++)
	{
	  BF_Dispose(&array[i]);
	}
      free(array);
      return KS;
    }
  else
    return 0;
}
