/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/statistic_tests.h"
#include "../include/bitutils.h"
#include "../include/special_functions.h"


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                         R U N    T E S T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct statistic Frequency(struct BitField *Field, int length)
type: low 
comment: This function compute the Run test for words of length given by the parameter length
*/
struct statistic Run(struct BitField *Field, int length, word_t mbit)
{
  bsize_t i, compteur, *array, n;
  word_t k;
  struct statistic result;
  double *proba;
  
  if (length >= WORDLENGTH || length==0)
    {
      result.error=1;
    return result;
    }
  array = (bsize_t *) calloc(length,sizeof(bsize_t));
  proba = (double *) calloc(length,sizeof(double));
  compteur=0;

  for (i=1; i <= BITLENGTH(Field); i++)
  {
    k = BF_Take(Field,i) & 1UL;
    if (k==mbit)
      {
	compteur++;
      }
    else
      {
	if (compteur > 0)
	  {
	    if (compteur <= length)
	      array[compteur-1]=array[compteur-1]+1;
	    compteur = 0;
	  }
      }
  }
  
  n=BITLENGTH(Field);
  for(i=0; i<length; i++)
    {
      proba[i]=((double)(n-i+2))/pow(2,(i+3));
    }

  result=Chi_Carre(array,proba,length-1);

  free(array);
  free(proba);
  return result;
}

/*
proto: struct test Statistic_Run(struct test_parameters parameters)
type: high
comment: This function computes the KolmogorovSmirvov test for the Run test
*/

struct test Statistic_Run(struct test_parameters parameters)
{
  struct test result;
  struct statistic chi2;
  struct BitField *array;
  double *proba;
  unsigned int nliberty,i;
  double KS, min, max;
  bsize_t offset;
  int flag;

  if (proba = (double *)calloc(parameters.nb_chunks, sizeof(double)))
    {
      array = BF_Split(parameters.data,parameters.nb_chunks);

#ifdef STATISTICS
      printf("Number of chunks : %d \n", parameters.nb_chunks);
      printf("Length of a chunk in bits : %d \n", BITLENGTH((&array[0])));
#endif

      min = HUGE_VAL; 
      max = -HUGE_VAL;
      flag=0;
      for(i=0; i<parameters.nb_chunks; i++)
	{
	  chi2=Run(&array[i],parameters.length, parameters.bit);
	  proba[i]=chi2.z;
	  flag= chi2.warning | flag;
	  if (min > proba[i])
	    min = proba[i];
	  if (max < proba[i])
	    max = proba[i];
	}
      nliberty=parameters.length - 1;
      KS = Kolmogorov_Smirnov(proba, parameters.nb_chunks, (double (*)(double, void *))chi2_distribution, &nliberty);

      min = MIN(parameters.min,min);
      max = MAX(parameters.max,max);

      if (strcmp(parameters.file_name,"") != 0)
	{
	  plot_graph(proba, parameters.nb_chunks, parameters.file_name);
	}
      if (strcmp(parameters.file_name1,"") != 0)
	{
      	 plot_function((double (*)(double, void *))chi2_distribution, &nliberty, min, max, parameters.nb_points, parameters.file_name1);
	}

      free(proba);
      offset=BITLENGTH(parameters.data)%parameters.nb_chunks;
      for(i=0; i<parameters.nb_chunks+(offset != 0); i++)
	{
	  BF_Dispose(&array[i]);
	}
      free(array);
      result.min = min;
      result.max = max;
      result.z= KS;
      result.error=0;
      result.warning = flag;
      return result;
    }
  else
    {
      result.error=1;
      result.z=0;
      return result;
    }
}
