/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/config.h"
#include "../include/statistic_tests.h"
#include "../include/bitutils.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                  S H A N O N   E N T R O P Y   T E S T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct test Shannon_Entropy(struct test_parameters parameters)
type: low 
comment: This function computes the Shannon Entropy 
*/
struct test Shannon_Entropy(struct test_parameters parameters)

{
  bsize_t i, length;
  bsize_t *array;
  word_t k;
  double proba, sum;
  struct test result;
  
  if (parameters.length>=32 || parameters.length==0) 
    {
      result.error=1;
      return result;
    }
  array = (bsize_t *) calloc((1<<parameters.length) ,sizeof(bsize_t));

  for (i=1; i <= BITLENGTH(parameters.data); i = i+parameters.length)
  {
    k = BF_Take(parameters.data,i) << (WORDLENGTH - parameters.length) >> (WORDLENGTH - parameters.length); 
    array[k]=array[k]+1;
  }
  length = BITLENGTH(parameters.data)/parameters.length;

#ifdef DEBUG
  for(i=0; i< (1<<parameters.length); i++)
    {
      printf("%d %d \n",i,array[i]); 
    }
#endif

  sum = 0;
  for (i=0; i< (1<<parameters.length); i++)
    {
      proba = (double)array[i]/length;
      if (proba != 0)
	sum = sum + proba * log(1/proba)/log(2);
      
      if (array[i] < LOW_SAMPLE)
	result.warning=1;
      
    }
  free(array);
  result.error=0;
  result.z = sum;
  return result;
}
