/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/config.h"
#include "../include/statistic_types.h"
#include "../include/bitutils.h"
#include "../include/special_functions.h"

extern double sqrt2( double);

/* for the purpose of Kolmogorov_Smirnov test we include here a slow sort algorithm */
void sort(double *array, unsigned int length)

{
  int flag;
  unsigned int i;
  double temp;
  
  flag = 1;
  
  while (flag == 1)
    {
      flag=0;
      for (i=0; i< length-1; i++)
	{
	  if (array[i] > array[i+1])
	    {
	      temp =array[i];
	      array[i]=array[i+1];
	      array[i+1]=temp;
	      flag=1;
	    }
	}
    }
}

/* functions to output as a plotable data file a graph of a function */
int plot_function(double (*F)(double,void *), void *parameter, double min, double max, unsigned int nb_points, char *name)

{
  unsigned int i;
  double step,x; 
  FILE *OUTPUT;
  
  if (OUTPUT=fopen(name,"w"))
    {
      if (max==min)
	return 0;
      step=(max-min)/nb_points;
      x=min;
      for (i=0; i<nb_points; i++)
	{
	  fprintf(OUTPUT,"%f %f \n",x,(*F)(x, parameter));
	  x=x+step;
    }
      fclose(OUTPUT);
      return 1;
    }
  else
    return 0;
}

/* functions to output as a plotable data file a graph of a function */
int plot_graph(double *array, unsigned int nb_points, char *name)

{
  unsigned int i;
  FILE *OUTPUT;
  
  sort(array, nb_points);
  
  if (OUTPUT=fopen(name, "w"))
    {
      for (i=0; i<nb_points; i++)
	{
	  fprintf(OUTPUT,"%f %f \n",array[i],(i+1.0)/nb_points);
	}
      fclose(OUTPUT);
      return 1;
    }
  else
    return 0;
}


/* this function compute the chi_carre statistic for the observations the probabilities probabilities and for nliberty degree of liberty */
struct statistic Chi_Carre(bsize_t* observations, double* probabilities, bsize_t nliberty)

{
  bsize_t i;
  double V;
  struct statistic result;

  result.warning=0;
  for (i=0; i<=nliberty; i++)
    {
      if (observations[i] < LOW_SAMPLE)
	result.warning=1;
    }
  V = 0;
  for (i=0; i<=nliberty; i++)
    {
      V = V + (double)(observations[i]-probabilities[i])*(observations[i]-probabilities[i])/(double)(probabilities[i]);
    }
  result.z= V;
  return result;
}

/* same as Chi_Carre calculated with an equiprobability for all the categories of observations */
struct statistic Equi_Chi_Carre(bsize_t* observations, bsize_t nliberty)

{
  bsize_t n,i;
  double V, nps;
  struct statistic result;

  n=0;
  for (i=0; i<=nliberty; i++)
    {
      n = n + observations[i];
      if (observations[i] < LOW_SAMPLE)
	result.warning=1;
    }
  V= 0;
  nps = (float)n/(nliberty+1);
  for (i=0; i<=nliberty; i++)
    {
      V = V + pow((double)(observations[i])-(double)nps,(double)2)/((double)nps);
    }
  result.z= V;
  return result;
}

/* this function returns the value of the Kolmogorov-Smirnov test */
double Kolmogorov_Smirnov(double *observations, unsigned int nb_observations, double (*F)(double,void *), void *parameter)

{
  double kplus, kmoins, kplusi, kmoinsi;
  unsigned int i;
  double z;

  sort(observations, nb_observations);
  kplus=-HUGE_VAL;
  kmoins=-HUGE_VAL;
  
  for (i=1; i<=nb_observations; i++)
    {
      kplusi= (double)i/(double)nb_observations-(*F)((double)observations[i-1], parameter);
      kmoinsi = (*F)((double)observations[i-1], parameter)-(i-1)/(double)nb_observations;

      if (kplus < kplusi) 
	kplus = kplusi;
      if (kmoins < kmoinsi) 
	kmoins = kmoinsi;
    }
  z = MAX(sqrt2(nb_observations)*kplus, sqrt2(nb_observations)*kmoins);
  return z;
}
