/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/***************************************************************/
/* This library contains a set of functions to perform all the */
/* bit manipulations currently used in alea testing and block  */
/* cypher simulation                                           */
/***************************************************************/
#ifndef _UTILITIES_H_
#define _UTILITIES_H_
#include "btypes.h"

/* low level function (which work on int) */
/* funtion which display in binary an int */
extern void word2string(word_t, bsize_t l, char *string, int base);

/* function which return a word composed of the l first bits of word1 and of the WODLENGTH-l bits of word 2 */
extern word_t wordcompose(word_t word1, word_t word2, bsize_t l);

/* function which return the weight of a word */
extern bsize_t wordweight(word_t);

/* function which return the scalar product of two words */
extern word_t wordscalar(word_t , word_t);

/* function which return the number of 1 minus the number of 0 of a word */
extern bsize_t wordbalance(word_t);

/* hight level functions */
/* function to create a BitField */
extern int BF_Create(struct BitField *Field, bsize_t l, bsize_t max);

/* function to realloc a BitField */
extern int BF_Realloc(struct BitField *Field, bsize_t l, bsize_t max);

/* function which split a BitField into n BitField of the same length plus possibly one more of leatest length*/
extern struct BitField *BF_Split(struct BitField *Field, int n);

/* function to read a BitField from a file*/
extern int BF_Open_File(struct BitField *Field, char *bitfile);

/* function to read a BitField from a file write ou 0 and 1 */
extern int BF_Open_Text_File(struct BitField *Field, char *bitfile, int base);

/* function to write a BitField to a file*/
extern int BF_WriteFile(struct BitField *Field, char *bitfile);

/* function which releases the resources devoted to a BitField */
extern void BF_Dispose (struct BitField *Field);

/* function to translate a BitField in a string representing its value in base base */
extern void BF_2_String(struct BitField *Field, bsize_t l, char *string, int base);

/* function which displays the l first bits of a BitField */
extern void BF_Display(struct BitField *Field, bsize_t l, int base);

/* function write to a file the l first bits of a BitField */
extern void BF_FDisplay(struct BitField *Field, bsize_t l, int base, FILE *output);

/* function which calculate the Walsh transform of the l first bits of a BitField */
extern int Fourrier_Word(long int (*f)(bsize_t, void *), unsigned int u, bsize_t l, void *parameter);

/* function which compute the Walsh transform of the function f */
extern long int BF_Fourrier_Word(long int (*f)(bsize_t, void *), word_t u, bsize_t l, void *parameter);

/* function which append one bit to a BitField */
extern void BF_Append_Bit(struct BitField *Field, unsigned int bit);

/* function which returns the weight of a BitField */
extern bsize_t BF_Weight(struct BitField *Field, bsize_t);

/* function which returns the scalar product of the first l bits of two BitField */
extern word_t BF_Scalar(struct BitField *Field1, struct BitField *Field2, bsize_t l);

/* function which return the number of 1 minus the number of 0 of a BitField */
extern void BF_Balance(struct BitField *Field, bsize_t l);

/* function which shift to the right a BitField and complete which the value of the int */
extern void BF_Right_Shift(struct BitField *Field, word_t bit);

/* function which shift to the left a BitField and complete which the value of the int */
extern void BF_Left_Shift(struct BitField *Field, word_t bit);

/* function which reverse the order of a BitField */
extern void BF_Reverse(struct BitField *Field1, struct BitField *Field2);

/* function which returns a word of a BitField of offset l */
extern word_t BF_Take(struct BitField *Field, bsize_t l);

/* function which set a word in a BitField at position l */
int BF_Put(struct BitField *Field, bsize_t l, word_t mword);

/* function which extract a Field of a BitField of offset l and length h*/
extern void BF_Take_Field(struct BitField *Field1, struct BitField *Field2, bsize_t l, bsize_t h);

/* function which returns the bit of a BitField of offset l*/
extern word_t BF_Take_Bit(struct BitField *Field, bsize_t l);

/* function which returns the offset of a chain of bits in a BitField which match a given pattern */
extern bsize_t BF_Match(struct BitField *Field, struct BitField *Match, int *flag);

/* function to compute the And of two BitFields */
extern void BF_And(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l);

/* function to compute the Or of two BitFields */
extern void BF_Or(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l);

/* function to compute the Xor of two BitFields */
extern void BF_Xor(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l);

#endif /* _UTILITIES_H_ */
